//+------------------------------------------------------------------+
//|                                                Adaptable_CCI.mq4 |
//|                               Copyright  2014, Gehtsoft USA LLC |
//|                                            http://fxcodebase.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2014, Gehtsoft USA LLC"
#property link      "http://fxcodebase.com"

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1 Yellow

extern string TimeFrame       = "Current time frame";
extern int Average_Length=14;
extern int Deviation_Length=14;
extern double Correction_Factor=0.015;
extern int Method=0;  // 0 - SMA
                      // 1 - EMA
                      // 2 - SMMA
                      // 3 - LWMA
extern int Price=0;    // Applied price
                       // 0 - Close
                       // 1 - Open
                       // 2 - High
                       // 3 - Low
                       // 4 - Median
                       // 5 - Typical
                       // 6 - Weighted
extern double Overbought_Level=100.;
extern double Oversold_Level=-100.;

double ACCI[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
int    timeFrame;

int init()
{
 
 IndicatorDigits(Digits);
 SetIndexStyle(0,DRAW_LINE);
 SetIndexBuffer(0,ACCI);
 
 SetLevelValue(0, Overbought_Level);
 SetLevelValue(1, Oversold_Level);
 
    indicatorFileName = WindowExpertName();
    returnBars        = TimeFrame == "returnBars";     if (returnBars)     return(0);
    timeFrame         = stringToTimeFrame(TimeFrame);
 IndicatorShortName(timeFrameToString(timeFrame)+" Adaptable CCI oscillator");
 return(0);
}

int deinit()
{

 return(0);
}

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { ACCI[0] = limit+1; return(0); }
            if (timeFrame!=Period())
            {
               limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
               for (int i=limit; i>=0; i--)
               {
                   int y = iBarShift(NULL,timeFrame,Time[i]);               
                      ACCI[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Average_Length,Deviation_Length,Correction_Factor,Method,Price,Overbought_Level,Oversold_Level,0,y);       
               }
            return(0);
      }

 if(Bars<=3) return(0);
 int ExtCountedBars=IndicatorCounted();
 if (ExtCountedBars<0) return(-1);
 limit=Bars-2;
 if(ExtCountedBars>2) limit=Bars-ExtCountedBars-1;
 int pos;
 double mean;
 double Avg;
 double Sum;
 double Pr;
 double meandev;
 pos=limit;
 while(pos>=0)
 {
  mean=iMA(NULL, 0, Average_Length, 0, Method, Price, pos);
  Avg=iMA(NULL, 0, Deviation_Length, 0, MODE_SMA, Price, pos);
  Sum=0.;
  for (i=0;i<Deviation_Length;i++)
  {
   Pr=iMA(NULL, 0, 1, 0, MODE_SMA, Price, pos+i);
   Sum=Sum+MathAbs(Pr-Avg);
  }
  meandev=Sum/Deviation_Length;
  
  Pr=iMA(NULL, 0, 1, 0, MODE_SMA, Price, pos);
  if (meandev==0.)
  {
   ACCI[pos]=0.;
  }
  else
  {
   ACCI[pos]=(Pr-mean)/(meandev*Correction_Factor);
  }

  pos--;
 } 
 return(0);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}


